@@ -4,6 +4,7 @@
  * SPDX-License-Identifier: LGPL-2.0-or-later
  *
  * SPDX-FileCopyrightText: 2016-2018 Jan Grulich <jgrulich@redhat.com>
+ * SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
  */
 
 #include "filechooser.h"
@@ -14,6 +15,7 @@
 #include <QDialogButtonBox>
 #include <QFile>
 #include <QFileDialog>
+#include <QFileInfo>
 #include <QGridLayout>
 #include <QLabel>
 #include <QLoggingCategory>
@@ -30,6 +32,9 @@
 #include <KSharedConfig>
 #include <KWindowConfig>
 
+#include "documents_interface.h"
+#include "fuse_interface.h"
+#include "request.h"
 #include <mobilefiledialog.h>
 
 Q_LOGGING_CATEGORY(XdgDesktopPortalKdeFileChooser, "xdp-kde-file-chooser")
@@ -131,6 +136,13 @@ const QDBusArgument &operator>>(const QDBusArgument &arg, FileChooserPortal::Opt
     return arg;
 }
 
+static bool isKIOFuseAvailable()
+{
+    static bool available =
+        QDBusConnection::sessionBus().interface() && QDBusConnection::sessionBus().interface()->activatableServiceNames().value().contains("org.kde.KIOFuse");
+    return available;
+}
+
 FileDialog::FileDialog(QDialog *parent, Qt::WindowFlags flags)
     : QDialog(parent, flags)
     , m_fileWidget(new KFileWidget(QUrl(), this))
@@ -184,6 +196,47 @@ FileChooserPortal::~FileChooserPortal()
 {
 }
 
+static QStringList fuseRedirect(QList<QUrl> urls)
+{
+    qCDebug(XdgDesktopPortalKdeFileChooser) << "mounting urls with fuse" << urls;
+
+    OrgKdeKIOFuseVFSInterface kiofuse_iface(QStringLiteral("org.kde.KIOFuse"), QStringLiteral("/org/kde/KIOFuse"), QDBusConnection::sessionBus());
+    struct MountRequest {
+        QDBusPendingReply<QString> reply;
+        int urlIndex;
+        QString basename;
+    };
+    QVector<MountRequest> requests;
+    requests.reserve(urls.count());
+    for (int i = 0; i < urls.count(); ++i) {
+        QUrl url = urls.at(i);
+        if (!url.isLocalFile()) {
+            const QString path(url.path());
+            const int slashes = path.count(QLatin1Char('/'));
+            QString basename;
+            if (slashes > 1) {
+                url.setPath(path.section(QLatin1Char('/'), 0, slashes - 1));
+                basename = path.section(QLatin1Char('/'), slashes, slashes);
+            }
+            requests.push_back({kiofuse_iface.mountUrl(url.toString()), i, basename});
+        }
+    }
+
+    for (auto &request : requests) {
+        request.reply.waitForFinished();
+        if (request.reply.isError()) {
+            qWarning() << "FUSE request failed:" << request.reply.error();
+            continue;
+        }
+
+        urls[request.urlIndex] = QUrl::fromLocalFile(request.reply.value() + QLatin1Char('/') + request.basename);
+    };
+
+    qCDebug(XdgDesktopPortalKdeFileChooser) << "mounted urls with fuse, maybe" << urls;
+
+    return QUrl::toStringList(urls);
+}
+
 uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
                                  const QString &app_id,
                                  const QString &parent_window,
@@ -255,7 +308,7 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
 
         uint retCode = m_mobileFileDialog->exec();
 
-        results.insert(QStringLiteral("uris"), m_mobileFileDialog->results());
+        results.insert(QStringLiteral("uris"), fuseRedirect(m_mobileFileDialog->results()));
 
         return retCode;
     }
@@ -268,13 +321,16 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
         dirDialog.setModal(modalDialog);
         dirDialog.setFileMode(QFileDialog::Directory);
         dirDialog.setOptions(QFileDialog::ShowDirsOnly);
-        dirDialog.setSupportedSchemes(QStringList{QStringLiteral("file")});
+        if (!isKIOFuseAvailable()) {
+            dirDialog.setSupportedSchemes(QStringList{QStringLiteral("file")});
+        }
         if (!acceptLabel.isEmpty()) {
             dirDialog.setLabelText(QFileDialog::Accept, acceptLabel);
         }
 
         dirDialog.winId(); // Trigger window creation
         Utils::setParentWindow(&dirDialog, parent_window);
+        Request::makeClosableDialogRequest(handle, &dirDialog);
 
         if (dirDialog.exec() != QDialog::Accepted) {
             return 1;
@@ -285,7 +341,7 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
             return 2;
         }
 
-        results.insert(QStringLiteral("uris"), QUrl::toStringList(urls, QUrl::FullyEncoded));
+        results.insert(QStringLiteral("uris"), fuseRedirect(urls));
         results.insert(QStringLiteral("writable"), true);
 
         return 0;
@@ -304,11 +360,14 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
 
     QScopedPointer<FileDialog, QScopedPointerDeleteLater> fileDialog(new FileDialog());
     Utils::setParentWindow(fileDialog.data(), parent_window);
+    Request::makeClosableDialogRequest(handle, fileDialog.get());
     fileDialog->setWindowTitle(title);
     fileDialog->setModal(modalDialog);
     KFile::Mode mode = directory ? KFile::Mode::Directory : multipleFiles ? KFile::Mode::Files : KFile::Mode::File;
     fileDialog->m_fileWidget->setMode(mode | KFile::Mode::ExistingOnly);
-    fileDialog->m_fileWidget->setSupportedSchemes(QStringList{QStringLiteral("file")});
+    if (!isKIOFuseAvailable()) {
+        fileDialog->m_fileWidget->setSupportedSchemes(QStringList{QStringLiteral("file")});
+    }
     fileDialog->m_fileWidget->okButton()->setText(!acceptLabel.isEmpty() ? acceptLabel : i18n("Open"));
 
     bool bMimeFilters = false;
@@ -330,7 +389,7 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
             return 2;
         }
 
-        results.insert(QStringLiteral("uris"), QUrl::toStringList(urls, QUrl::FullyEncoded));
+        results.insert(QStringLiteral("uris"), fuseRedirect(urls));
         results.insert(QStringLiteral("writable"), true);
 
         if (optionsWidget) {
@@ -355,6 +414,59 @@ uint FileChooserPortal::OpenFile(const QDBusObjectPath &handle,
     return 1;
 }
 
+enum class Entity { File, Folder };
+static QUrl kioUrlFromSandboxPath(const QString &path, Entity entity)
+{
+    if (path.isEmpty()) {
+        return {};
+    }
+
+    static QString mountPoint;
+    if (!mountPoint.isEmpty() && !path.startsWith(mountPoint)) {
+        return QUrl::fromLocalFile(path);
+    }
+
+    OrgFreedesktopPortalDocumentsInterface documents_iface(QStringLiteral("org.freedesktop.portal.Documents"),
+                                                           QStringLiteral("/org/freedesktop/portal/documents"),
+                                                           QDBusConnection::sessionBus());
+
+    if (mountPoint.isEmpty()) {
+        mountPoint = QString::fromUtf8(documents_iface.GetMountPoint());
+        if (!path.startsWith(mountPoint)) {
+            return QUrl::fromLocalFile(path);
+        }
+    }
+
+    QByteArray localFilePath;
+    switch (entity) {
+    case Entity::File: // basename of dirpath (= id of the shared document on the portal)
+        localFilePath = documents_iface.Info(QFileInfo(QFileInfo(path).path()).fileName());
+        break;
+    case Entity::Folder: // basename of path
+        localFilePath = documents_iface.Info(QFileInfo(path).fileName());
+        break;
+    }
+    if (localFilePath.isEmpty()) {
+        return QUrl::fromLocalFile(path);
+    }
+
+    if (!isKIOFuseAvailable()) {
+        return QUrl::fromLocalFile(localFilePath);
+    }
+
+    OrgKdeKIOFuseVFSInterface fuse_iface(QStringLiteral("org.kde.KIOFuse"), QStringLiteral("/org/kde/KIOFuse"), QDBusConnection::sessionBus());
+
+    QString remoteFilePath = fuse_iface.remoteUrl(localFilePath);
+    if (remoteFilePath.isEmpty()) {
+        return QUrl::fromLocalFile(path);
+    }
+
+    QUrl url(remoteFilePath);
+    url.setPath(QFileInfo(url.path()).path());
+    qCDebug(XdgDesktopPortalKdeFileChooser) << "Translated portal url to" << url;
+    return url;
+};
+
 uint FileChooserPortal::SaveFile(const QDBusObjectPath &handle,
                                  const QString &app_id,
                                  const QString &parent_window,
@@ -436,7 +548,7 @@ uint FileChooserPortal::SaveFile(const QDBusObjectPath &handle,
 
         uint retCode = m_mobileFileDialog->exec();
 
-        results.insert(QStringLiteral("uris"), m_mobileFileDialog->results());
+        results.insert(QStringLiteral("uris"), fuseRedirect(m_mobileFileDialog->results()));
 
         return retCode;
     }
@@ -454,17 +566,27 @@ uint FileChooserPortal::SaveFile(const QDBusObjectPath &handle,
 
     QScopedPointer<FileDialog, QScopedPointerDeleteLater> fileDialog(new FileDialog());
     Utils::setParentWindow(fileDialog.data(), parent_window);
+    Request::makeClosableDialogRequest(handle, fileDialog.get());
     fileDialog->setWindowTitle(title);
     fileDialog->setModal(modalDialog);
     fileDialog->m_fileWidget->setOperationMode(KFileWidget::Saving);
     fileDialog->m_fileWidget->setConfirmOverwrite(true);
 
-    if (!currentFolder.isEmpty()) {
-        fileDialog->m_fileWidget->setUrl(QUrl::fromLocalFile(currentFolder));
+    const QUrl translatedCurrentFolderUrl = kioUrlFromSandboxPath(currentFolder, Entity::Folder);
+    if (!translatedCurrentFolderUrl.isEmpty()) {
+        fileDialog->m_fileWidget->setUrl(translatedCurrentFolderUrl);
     }
 
     if (!currentFile.isEmpty()) {
-        fileDialog->m_fileWidget->setSelectedUrl(QUrl::fromLocalFile(currentFile));
+        // If we also had a currentfolder then recycle its URL, otherwise calculate from scratch.
+        // In either case append the basename to get to a complete file URL again.
+        QUrl kioUrl = translatedCurrentFolderUrl.isEmpty() ? kioUrlFromSandboxPath(currentFile, Entity::File) : translatedCurrentFolderUrl;
+        if (!kioUrl.isEmpty()) {
+            kioUrl.setPath(kioUrl.path() + QLatin1Char('/') + QFileInfo(currentFile).completeBaseName());
+            fileDialog->m_fileWidget->setSelectedUrl(kioUrl);
+        } else {
+            fileDialog->m_fileWidget->setSelectedUrl(QUrl::fromLocalFile(currentFile));
+        }
     }
 
     if (!currentName.isEmpty()) {
@@ -489,10 +611,8 @@ uint FileChooserPortal::SaveFile(const QDBusObjectPath &handle,
     }
 
     if (fileDialog->exec() == QDialog::Accepted) {
-        QStringList files;
-        QUrl url = QUrl::fromLocalFile(fileDialog->m_fileWidget->selectedFile());
-        files << url.toDisplayString();
-        results.insert(QStringLiteral("uris"), files);
+        const auto urls = fileDialog->m_fileWidget->selectedUrls();
+        results.insert(QStringLiteral("uris"), fuseRedirect(urls));
 
         if (optionsWidget) {
             QVariant choices = EvaluateSelectedChoices(checkboxes, comboboxes);
