@@ -4,10 +4,12 @@
  * SPDX-License-Identifier: LGPL-2.0-or-later
  *
  * SPDX-FileCopyrightText: 2018 Jan Grulich <jgrulich@redhat.com>
+ * SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
  */
 
 #include "remotedesktop.h"
 #include "remotedesktopdialog.h"
+#include "request.h"
 #include "session.h"
 #include "utils.h"
 #include "waylandintegration.h"
@@ -117,19 +119,26 @@ uint RemoteDesktopPortal::Start(const QDBusObjectPath &handle,
     QScopedPointer<RemoteDesktopDialog, QScopedPointerDeleteLater> remoteDesktopDialog(
         new RemoteDesktopDialog(app_id, session->deviceTypes(), session->screenSharingEnabled(), session->multipleSources()));
     Utils::setParentWindow(remoteDesktopDialog->windowHandle(), parent_window);
+    Request::makeClosableDialogRequest(handle, remoteDesktopDialog.get());
 
     connect(session, &Session::closed, remoteDesktopDialog.data(), &RemoteDesktopDialog::reject);
 
     if (remoteDesktopDialog->exec()) {
         if (session->screenSharingEnabled()) {
-            auto stream = WaylandIntegration::startStreamingOutput(remoteDesktopDialog->selectedScreens().first(), Screencasting::Hidden);
-            if (!stream.isValid()) {
+            if (!WaylandIntegration::startStreamingOutput(remoteDesktopDialog->selectedOutputs().constFirst().waylandOutputName(), Screencasting::Hidden)) {
                 return 2;
             }
 
             WaylandIntegration::authenticate();
 
-            results.insert(QStringLiteral("streams"), QVariant::fromValue<WaylandIntegration::Streams>({stream}));
+            QVariant streams = WaylandIntegration::streams();
+
+            if (!streams.isValid()) {
+                qCWarning(XdgDesktopPortalKdeRemoteDesktop()) << "Pipewire stream is not ready to be streamed";
+                return 2;
+            }
+
+            results.insert(QStringLiteral("streams"), streams);
         } else {
             qCWarning(XdgDesktopPortalKdeRemoteDesktop()) << "Only stream input";
             WaylandIntegration::startStreamingInput();
