@@ -7,7 +7,6 @@
  */
 
 #include "remotedesktopdialog.h"
-#include "outputsmodel.h"
 
 #include "utils.h"
 #include <KLocalizedString>
@@ -26,7 +25,8 @@ RemoteDesktopDialog::RemoteDesktopDialog(const QString &appName,
                                          QObject *parent)
     : QuickDialog(parent)
 {
-    auto model = new OutputsModel(this);
+    // We disable sharing the full workspace, can be addressed eventually
+    auto model = new OutputsModel(OutputsModel::None, this);
 
     QVariantMap props = {
         {"outputsModel", QVariant::fromValue<QObject *>(model)},
@@ -34,7 +34,7 @@ RemoteDesktopDialog::RemoteDesktopDialog(const QString &appName,
         {"withMultipleScreenSharing", multiple},
         {"withKeyboard", deviceTypes.testFlag(RemoteDesktopPortal::Keyboard)},
         {"withPointer", deviceTypes.testFlag(RemoteDesktopPortal::Pointer)},
-        {"withTouch", deviceTypes.testFlag(RemoteDesktopPortal::TouchScreen)},
+        {"withTouchScreen", deviceTypes.testFlag(RemoteDesktopPortal::TouchScreen)},
     };
 
     const QString applicationName = Utils::applicationName(appName);
@@ -47,13 +47,13 @@ RemoteDesktopDialog::RemoteDesktopDialog(const QString &appName,
     create(QStringLiteral("qrc:/RemoteDesktopDialog.qml"), props);
 }
 
-QList<quint32> RemoteDesktopDialog::selectedScreens() const
+QList<Output> RemoteDesktopDialog::selectedOutputs() const
 {
     OutputsModel *model = dynamic_cast<OutputsModel *>(m_theDialog->property("outputsModel").value<QObject *>());
     if (!model) {
         return {};
     }
-    return model->selectedScreens();
+    return model->selectedOutputs();
 }
 
 RemoteDesktopPortal::DeviceTypes RemoteDesktopDialog::deviceTypes() const
@@ -63,7 +63,7 @@ RemoteDesktopPortal::DeviceTypes RemoteDesktopDialog::deviceTypes() const
         types |= RemoteDesktopPortal::Keyboard;
     if (m_theDialog->property("withPointer").toBool())
         types |= RemoteDesktopPortal::Pointer;
-    if (m_theDialog->property("withTouch").toBool())
+    if (m_theDialog->property("withTouchScreen").toBool())
         types |= RemoteDesktopPortal::TouchScreen;
     return types;
 }
