@@ -14,7 +14,6 @@
 
 #include "remotedesktop.h"
 #include "screencast.h"
-#include "waylandintegration.h"
 
 class Session : public QDBusVirtualObject
 {
@@ -62,22 +61,22 @@ public:
     {
         return SessionType::ScreenCast;
     }
-
-    WaylandIntegration::Streams streams() const
+    QVariant restoreData() const
     {
-        return m_streams;
+        return m_restoreData;
     }
-    void setStreams(const WaylandIntegration::Streams &streams)
+    ScreenCastPortal::PersistMode persistMode() const
     {
-        m_streams = streams;
+        return m_persistMode;
     }
 
 private:
     bool m_multipleSources;
     ScreenCastPortal::CursorModes m_cursorMode;
     ScreenCastPortal::SourceType m_types;
+    ScreenCastPortal::PersistMode m_persistMode = ScreenCastPortal::NoPersist;
 
-    WaylandIntegration::Streams m_streams;
+    QVariant m_restoreData;
 };
 
 class RemoteDesktopSession : public ScreenCastSession
