@@ -17,12 +17,12 @@
 
 #include <KNotification>
 #include <QEventLoop>
+#include <QImage>
 #include <QLoggingCategory>
+#include <QScreen>
 #include <QThread>
 #include <QTimer>
 
-#include <QImage>
-
 #include <KLocalizedString>
 
 // KWayland
@@ -31,6 +31,8 @@
 #include <KWayland/Client/plasmawindowmanagement.h>
 #include <KWayland/Client/plasmawindowmodel.h>
 #include <KWayland/Client/registry.h>
+#include <KWayland/Client/surface.h>
+#include <KWayland/Client/xdgforeign.h>
 
 // system
 #include <fcntl.h>
@@ -43,7 +45,7 @@ Q_LOGGING_CATEGORY(XdgDesktopPortalKdeWaylandIntegration, "xdp-kde-wayland-integ
 
 Q_GLOBAL_STATIC(WaylandIntegration::WaylandIntegrationPrivate, globalWaylandIntegration)
 
-static QDebug operator<<(QDebug dbg, const WaylandIntegration::Stream &c)
+static QDebug operator<<(QDebug dbg, const WaylandIntegration::WaylandIntegrationPrivate::Stream &c)
 {
     dbg.nospace() << "Stream(" << c.map << ", " << c.nodeId << ")";
     return dbg.space();
@@ -74,12 +76,22 @@ void WaylandIntegration::startStreamingInput()
     globalWaylandIntegration->startStreamingInput();
 }
 
-WaylandIntegration::Stream WaylandIntegration::startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode)
+bool WaylandIntegration::startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode)
 {
     return globalWaylandIntegration->startStreamingOutput(outputName, mode);
 }
 
-WaylandIntegration::Stream WaylandIntegration::startStreamingWindow(const QMap<int, QVariant> &win)
+bool WaylandIntegration::startStreamingWorkspace(Screencasting::CursorMode mode)
+{
+    return globalWaylandIntegration->startStreamingWorkspace(mode);
+}
+
+bool WaylandIntegration::startStreamingVirtual(const QString &name, const QSize &size, Screencasting::CursorMode mode)
+{
+    return globalWaylandIntegration->startStreamingVirtualOutput(name, size, mode);
+}
+
+bool WaylandIntegration::startStreamingWindow(const QMap<int, QVariant> &win)
 {
     return globalWaylandIntegration->startStreamingWindow(win);
 }
@@ -89,11 +101,6 @@ void WaylandIntegration::stopAllStreaming()
     globalWaylandIntegration->stopAllStreaming();
 }
 
-void WaylandIntegration::stopStreaming(uint node)
-{
-    globalWaylandIntegration->stopStreaming(node);
-}
-
 void WaylandIntegration::requestPointerButtonPress(quint32 linuxButton)
 {
     globalWaylandIntegration->requestPointerButtonPress(linuxButton);
@@ -129,6 +136,16 @@ QMap<quint32, WaylandIntegration::WaylandOutput> WaylandIntegration::screens()
     return globalWaylandIntegration->screens();
 }
 
+QVariant WaylandIntegration::streams()
+{
+    return globalWaylandIntegration->streams();
+}
+
+void WaylandIntegration::setParentWindow(QWindow *window, const QString &parentWindow)
+{
+    globalWaylandIntegration->setParentWindow(window, parentWindow);
+}
+
 // Thank you kscreen
 void WaylandIntegration::WaylandOutput::setOutputType(const QString &type)
 {
@@ -157,9 +174,7 @@ void WaylandIntegration::WaylandOutput::setOutputType(const QString &type)
     }
 }
 
-namespace WaylandIntegration
-{
-const QDBusArgument &operator>>(const QDBusArgument &arg, Stream &stream)
+const QDBusArgument &operator>>(const QDBusArgument &arg, WaylandIntegration::WaylandIntegrationPrivate::Stream &stream)
 {
     arg.beginStructure();
     arg >> stream.nodeId;
@@ -179,7 +194,7 @@ const QDBusArgument &operator>>(const QDBusArgument &arg, Stream &stream)
     return arg;
 }
 
-const QDBusArgument &operator<<(QDBusArgument &arg, const Stream &stream)
+const QDBusArgument &operator<<(QDBusArgument &arg, const WaylandIntegration::WaylandIntegrationPrivate::Stream &stream)
 {
     arg.beginStructure();
     arg << stream.nodeId;
@@ -188,7 +203,9 @@ const QDBusArgument &operator<<(QDBusArgument &arg, const Stream &stream)
 
     return arg;
 }
-}
+
+Q_DECLARE_METATYPE(WaylandIntegration::WaylandIntegrationPrivate::Stream)
+Q_DECLARE_METATYPE(WaylandIntegration::WaylandIntegrationPrivate::Streams)
 
 KWayland::Client::PlasmaWindowManagement *WaylandIntegration::plasmaWindowManagement()
 {
@@ -207,8 +224,8 @@ WaylandIntegration::WaylandIntegrationPrivate::WaylandIntegrationPrivate()
     , m_fakeInput(nullptr)
     , m_screencasting(nullptr)
 {
-    qDBusRegisterMetaType<Stream>();
-    qDBusRegisterMetaType<Streams>();
+    qDBusRegisterMetaType<WaylandIntegrationPrivate::Stream>();
+    qDBusRegisterMetaType<WaylandIntegrationPrivate::Streams>();
 }
 
 WaylandIntegration::WaylandIntegrationPrivate::~WaylandIntegrationPrivate() = default;
@@ -228,25 +245,66 @@ void WaylandIntegration::WaylandIntegrationPrivate::startStreamingInput()
     m_streamInput = true;
 }
 
-WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingWindow(const QMap<int, QVariant> &win)
+bool WaylandIntegration::WaylandIntegrationPrivate::startStreamingWindow(const QMap<int, QVariant> &win)
 {
     auto uuid = win[KWayland::Client::PlasmaWindowModel::Uuid].toString();
-    return startStreaming(m_screencasting->createWindowStream(uuid, Screencasting::Hidden), {}, win);
+    QString iconName = win[Qt::DecorationRole].value<QIcon>().name();
+    iconName = iconName.isEmpty() ? QStringLiteral("applications-all") : iconName;
+    return startStreaming(m_screencasting->createWindowStream(uuid, Screencasting::Hidden),
+                          iconName,
+                          i18n("Recording window \"%1\"...", win[Qt::DisplayRole].toString()),
+                          {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}});
 }
 
-WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode)
+bool WaylandIntegration::WaylandIntegrationPrivate::startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode)
 {
     auto output = m_outputMap.value(outputName).output();
-
-    return startStreaming(m_screencasting->createOutputStream(output.data(), mode), output, {});
+    m_streamedScreenPosition = output->globalPosition();
+    return startStreaming(m_screencasting->createOutputStream(output.data(), mode),
+                          QStringLiteral("video-display"),
+                          i18n("Recording screen \"%1\"...", output->model()),
+                          {
+                              {QLatin1String("size"), output->pixelSize()},
+                              {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Monitor)},
+                          });
 }
 
-WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startStreaming(ScreencastingStream *stream,
-                                                                                         QSharedPointer<KWayland::Client::Output> output,
-                                                                                         const QMap<int, QVariant> &win)
+bool WaylandIntegration::WaylandIntegrationPrivate::startStreamingWorkspace(Screencasting::CursorMode mode)
+{
+    QRect workspace;
+    const auto screens = qGuiApp->screens();
+    for (QScreen *screen : screens) {
+        workspace |= screen->geometry();
+    }
+    m_streamedScreenPosition = workspace.topLeft();
+    return startStreaming(m_screencasting->createRegionStream(workspace, 1, mode),
+                          QStringLiteral("video-display"),
+                          i18n("Recording workspace..."),
+                          {
+                              {QLatin1String("size"), workspace.size()},
+                              {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Monitor)},
+                          });
+}
+
+bool WaylandIntegration::WaylandIntegrationPrivate::startStreamingVirtualOutput(const QString &name, const QSize &size, Screencasting::CursorMode mode)
+{
+    return startStreaming(m_screencasting->createVirtualOutputStream(name, size, 1, mode),
+                          QStringLiteral("video-display"),
+                          i18n("Recording virtual output '%1'...", name),
+                          {
+                              {QLatin1String("size"), size},
+                              {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Virtual)},
+                          });
+}
+
+bool WaylandIntegration::WaylandIntegrationPrivate::startStreaming(ScreencastingStream *stream,
+                                                                   const QString &iconName,
+                                                                   const QString &description,
+                                                                   const QVariantMap &streamOptions)
 {
     QEventLoop loop;
-    Stream ret;
+    bool streamReady = false;
+
     connect(stream, &ScreencastingStream::failed, this, [&](const QString &error) {
         qCWarning(XdgDesktopPortalKdeWaylandIntegration) << "failed to start streaming" << stream << error;
 
@@ -255,38 +313,29 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
         notification->setText(error);
         notification->setIconName(QStringLiteral("dialog-error"));
         notification->sendEvent();
+
+        streamReady = false;
         loop.quit();
     });
     connect(stream, &ScreencastingStream::created, this, [&](uint32_t nodeid) {
-        ret.stream = stream;
-        ret.nodeId = nodeid;
-        if (output) {
-            m_streamedScreenPosition = output->globalPosition();
-            ret.map = {
-                {QLatin1String("size"), output->pixelSize()},
-                {QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Monitor)},
-            };
-        } else {
-            ret.map = {{QLatin1String("source_type"), static_cast<uint>(ScreenCastPortal::Window)}};
-        }
-        m_streams.append(ret);
+        Stream s;
+        s.stream = stream;
+        s.nodeId = nodeid;
+        s.map = streamOptions;
+        m_streams.append(s);
         startStreamingInput();
 
         connect(stream, &ScreencastingStream::closed, this, [this, nodeid] {
             stopStreaming(nodeid);
         });
+        streamReady = true;
+        qDebug() << "start streaming" << s << m_streamedScreenPosition;
 
         auto item = new KStatusNotifierItem(stream);
         item->setStandardActionsEnabled(false);
-        if (output) {
-            item->setTitle(i18n("Recording screen \"%1\"...", output->model()));
-            item->setIconByName("video-display");
-        } else {
-            auto name = win[Qt::DecorationRole].value<QIcon>().name();
-            item->setIconByName(name.isEmpty() ? "applications-all" : name);
-            item->setTitle(i18n("Recording window \"%1\"...", win[Qt::DisplayRole].toString()));
-        }
-        item->setOverlayIconByName("media-record");
+        item->setTitle(description);
+        item->setIconByName(iconName);
+        item->setOverlayIconByName(QStringLiteral("media-record"));
         item->setToolTip(item->iconName(), item->title(), i18n("Press to cancel"));
         item->setStatus(KStatusNotifierItem::Active);
         connect(item, &KStatusNotifierItem::activateRequested, stream, [=] {
@@ -299,10 +348,10 @@ WaylandIntegration::Stream WaylandIntegration::WaylandIntegrationPrivate::startS
     QTimer::singleShot(3000, &loop, &QEventLoop::quit);
     loop.exec();
 
-    return ret;
+    return streamReady;
 }
 
-void WaylandIntegration::Stream::close()
+void WaylandIntegration::WaylandIntegrationPrivate::Stream::close()
 {
     stream->deleteLater();
 }
@@ -324,7 +373,6 @@ void WaylandIntegration::WaylandIntegrationPrivate::stopStreaming(uint32_t nodei
 {
     for (auto it = m_streams.begin(), itEnd = m_streams.end(); it != itEnd; ++it) {
         if (it->nodeId == nodeid) {
-            it->close();
             m_streams.erase(it);
             break;
         }
@@ -386,10 +434,44 @@ QMap<quint32, WaylandIntegration::WaylandOutput> WaylandIntegration::WaylandInte
     return m_outputMap;
 }
 
+QVariant WaylandIntegration::WaylandIntegrationPrivate::streams()
+{
+    return QVariant::fromValue<WaylandIntegrationPrivate::Streams>(m_streams);
+}
+
+static const char *windowParentHandlePropertyName = "waylandintegration-parentHandle";
+void WaylandIntegration::WaylandIntegrationPrivate::setParentWindow(QWindow *window, const QString &parentHandle)
+{
+    if (!m_xdgImporter) {
+        return;
+    }
+
+    if (window->isVisible()) {
+        auto importedParent = m_xdgImporter->importTopLevel(parentHandle, window);
+        auto surface = KWayland::Client::Surface::fromWindow(window);
+        importedParent->setParentOf(surface);
+    }
+
+    window->setProperty(windowParentHandlePropertyName, parentHandle);
+    window->installEventFilter(this);
+}
+
+bool WaylandIntegration::WaylandIntegrationPrivate::eventFilter(QObject *watched, QEvent *event)
+{
+    const bool ret = WaylandIntegration::WaylandIntegration::eventFilter(watched, event);
+    QWindow *window = static_cast<QWindow *>(watched);
+    if (event->type() == QEvent::Expose && window->isExposed()) {
+        const QString parentHandle = window->property(windowParentHandlePropertyName).toString();
+        auto importedParent = m_xdgImporter->importTopLevel(parentHandle, window);
+        importedParent->setParentOf(KWayland::Client::Surface::fromWindow(window));
+    }
+    return ret;
+}
+
 void WaylandIntegration::WaylandIntegrationPrivate::authenticate()
 {
     if (!m_waylandAuthenticationRequested) {
-        m_fakeInput->authenticate(i18n("xdg-desktop-portals-kde"), i18n("Remote desktop"));
+        m_fakeInput->authenticate(QStringLiteral("xdg-desktop-portals-kde"), i18n("Remote desktop"));
         m_waylandAuthenticationRequested = true;
     }
 }
@@ -452,7 +534,9 @@ void WaylandIntegration::WaylandIntegrationPrivate::initWayland()
         m_windowManagement = m_registry->createPlasmaWindowManagement(name, version, this);
         Q_EMIT waylandIntegration()->plasmaWindowManagementInitialized();
     });
-
+    connect(m_registry, &KWayland::Client::Registry::importerUnstableV2Announced, this, [this](quint32 name, quint32 version) {
+        m_xdgImporter = m_registry->createXdgImporter(name, std::min(version, quint32(1)), this);
+    });
     connect(m_registry, &KWayland::Client::Registry::interfacesAnnounced, this, [this] {
         m_registryInitialized = true;
         qCDebug(XdgDesktopPortalKdeWaylandIntegration) << "Registry initialized";
