@@ -9,7 +9,6 @@
 #ifndef XDG_DESKTOP_PORTAL_KDE_WAYLAND_INTEGRATION_H
 #define XDG_DESKTOP_PORTAL_KDE_WAYLAND_INTEGRATION_H
 
-#include <QDBusArgument>
 #include <QObject>
 #include <QPoint>
 #include <QSize>
@@ -27,25 +26,10 @@ class ScreencastingSource;
 }
 }
 
+class QWindow;
+
 namespace WaylandIntegration
 {
-
-struct Stream {
-    ScreencastingStream *stream = nullptr;
-    uint nodeId;
-    QVariantMap map;
-
-    bool isValid() const
-    {
-        return stream != nullptr;
-    }
-
-    void close();
-};
-typedef QVector<Stream> Streams;
-const QDBusArgument &operator<<(QDBusArgument &arg, const Stream &stream);
-const QDBusArgument &operator>>(const QDBusArgument &arg, Stream &stream);
-
 class WaylandOutput
 {
 public:
@@ -53,6 +37,8 @@ public:
         Laptop,
         Monitor,
         Television,
+        Workspace,
+        Virtual,
     };
     QString manufacturer() const
     {
@@ -128,10 +114,11 @@ bool isStreamingEnabled();
 bool isStreamingAvailable();
 
 void startStreamingInput();
-Stream startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode);
-Stream startStreamingWindow(const QMap<int, QVariant> &win);
+bool startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode);
+bool startStreamingWorkspace(Screencasting::CursorMode mode);
+bool startStreamingVirtual(const QString &name, const QSize &size, Screencasting::CursorMode mode);
+bool startStreamingWindow(const QMap<int, QVariant> &win);
 void stopAllStreaming();
-void stopStreaming(uint node);
 
 void requestPointerButtonPress(quint32 linuxButton);
 void requestPointerButtonRelease(quint32 linuxButton);
@@ -142,6 +129,9 @@ void requestPointerAxisDiscrete(Qt::Orientation axis, qreal delta);
 void requestKeyboardKeycode(int keycode, bool state);
 
 QMap<quint32, WaylandOutput> screens();
+QVariant streams();
+
+void setParentWindow(QWindow *window, const QString &parentWindow);
 
 void init();
 
@@ -150,7 +140,4 @@ KWayland::Client::PlasmaWindowManagement *plasmaWindowManagement();
 WaylandIntegration *waylandIntegration();
 }
 
-Q_DECLARE_METATYPE(WaylandIntegration::Stream)
-Q_DECLARE_METATYPE(WaylandIntegration::Streams)
-
 #endif // XDG_DESKTOP_PORTAL_KDE_WAYLAND_INTEGRATION_H
