@@ -30,9 +30,12 @@ class PlasmaWindowManagement;
 class FakeInput;
 class RemoteBuffer;
 class Output;
+class XdgImporter;
 }
 }
 
+class QWindow;
+
 namespace WaylandIntegration
 {
 class WaylandIntegrationPrivate : public WaylandIntegration::WaylandIntegration
@@ -51,8 +54,18 @@ private:
 
     KWayland::Client::Registry *m_registry = nullptr;
     KWayland::Client::PlasmaWindowManagement *m_windowManagement = nullptr;
+    KWayland::Client::XdgImporter *m_xdgImporter = nullptr;
 
 public:
+    struct Stream {
+        ScreencastingStream *stream = nullptr;
+        uint nodeId;
+        QVariantMap map;
+
+        void close();
+    };
+    typedef QVector<Stream> Streams;
+
     void authenticate();
 
     bool isStreamingEnabled() const;
@@ -60,9 +73,11 @@ public:
 
     void startStreamingInput();
 
-    Stream startStreaming(ScreencastingStream *stream, QSharedPointer<KWayland::Client::Output> output, const QMap<int, QVariant> &win);
-    Stream startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode);
-    Stream startStreamingWindow(const QMap<int, QVariant> &win);
+    bool startStreaming(ScreencastingStream *stream, const QString &iconName, const QString &description, const QVariantMap &streamOptions);
+    bool startStreamingOutput(quint32 outputName, Screencasting::CursorMode mode);
+    bool startStreamingWindow(const QMap<int, QVariant> &win);
+    bool startStreamingWorkspace(Screencasting::CursorMode mode);
+    bool startStreamingVirtualOutput(const QString &name, const QSize &size, Screencasting::CursorMode mode);
     void stopStreaming(uint32_t nodeid);
     void stopAllStreaming();
 
@@ -76,11 +91,15 @@ public:
     QMap<quint32, WaylandOutput> screens();
     QVariant streams();
 
+    void setParentWindow(QWindow *window, const QString &parentHandle);
+
 protected Q_SLOTS:
     void addOutput(quint32 name, quint32 version);
     void removeOutput(quint32 name);
 
 private:
+    bool eventFilter(QObject *watched, QEvent *event) override;
+
     bool m_streamInput = false;
     bool m_waylandAuthenticationRequested = false;
 
